import pymongo as pym
import json
import os

# add a possible check for dictionaries

MONGO_URL = os.getenv('MONGO_URL', 'mongodb://localhost:27017')
MONGO_USERNAME = os.getenv('MONGO_USERNAME', 'default-user')
MONGO_PASSWORD = os.getenv('MONGO_PASSWORD', 'default_password')
MONGO_TLS = os.getenv('MONGO_TLS', 'global-bundle.pem')

CONNECT_URL = f'mongodb://{MONGO_USERNAME}:{MONGO_PASSWORD}@{MONGO_URL}?tls=true&tlsCAFile={MONGO_TLS}&retryWrites=false'

def import_to_documentdb(file):
    try:
        client = pym.MongoClient(CONNECT_URL) 

        db = client['database']
        collection = db['collection']
    except pym.errors.ConnectionFailure as err:
        print(f'Unable to connect to MongoDB: {err}')
        return None

    with open(file, 'r') as data:
        imported_data = json.load(data)

    if isinstance(imported_data, list):
        try:
            collection.insert_many(imported_data)
            print('Successfully inserted multiple collections.')
        except Exception as ex:
            print(f'Unable to insert documents: {ex}')
    else:
        try:
            collection.insert_one(imported_data)
            print('Successfully inserted singular collection.')
        except Exception as ex:
            print(f'Unable to insert document: {ex}')

    print('Now printing document to the terminal.')

    try:
        doc_doc = collection.find()
        for doc in doc_doc:
            print(doc)
    except Exception as err:
        print(f'Cannot fetch documents: {err}')

    print('Data has been imported to DocumentDB.')
    client.close()


# sanitization of csv/txt files

# goals of sanitization
# 1) identify primary key and flag duplicates
# 2) flag nulls
# 3) data type consistency ????
# 4) sanitize special characters such as commas, quotes, etc.
# 5) validate header names against meta data from db ????

# can use either pandas or csv library, pandas gives control over larger
# data sets while csv library gives more precise control and is used for
# smaller data sets


##########################################################################


# Importing Pandas library for large-scale data manipulation
import pandas as pd 

# Setting variable for file path
file = "./monsh2.csv"

# Setting data frame for file to break the CSV down into parsable data
d_frame = pd.read_csv(file)

def primary_key_sanitization(d_frame):
    try:
        col_num = int(input("Enter the column number corresponding to the primary key of your data: "))
        if col_num < 0 or col_num >= len(d_frame.columns):
            raise IndexError('Invalid column number.')
    except IndexError as err:
        print(err)
        return
    
    p_key = d_frame.columns[col_num]
    
    dupli_check = d_frame[d_frame.duplicated(subset=[p_key], keep=False)]
    
    if not dupli_check.empty:
        print(f"\nDuplicates found in the specified column '{p_key}':")
        print(dupli_check)
    else:
        print(f"\nNo duplicates found in specified column.")
    
    return d_frame

def identify_nulls(d_frame):
    null_sum = d_frame.isnull().sum()
    
    print("Null values within each column:")
    print(null_sum)
    
    tuple_nulls = d_frame[d_frame.isnull().any(axis=1)]
    
    if not tuple_nulls.empty:
        print("\nRows containing null values:")
        print(tuple_nulls)
    else: 
        print("\nNo rows containing null values.")
    
    return d_frame

# def special_character_cleanse(d_frame):
    
    
d_frame = primary_key_sanitization(d_frame)
d_frame = identify_nulls(d_frame)  

# print(d_frame.head(10))


import csv
import pandas as pd
import boto3
import requests
import json
import os

NEP_ENDPOINT = os.getenv('NEP_ENDPOINT', 'db-neptune-1.cluster-cfsssmgsia9l.us-east-1.neptune.amazonaws.com')
IAM_ROLE_ARN = os.getenv('IAM_ROLE_ARN', 'arn:aws:iam::123456789012:role/NeptuneAccessDev')
IMPORT_BUCKET = os.getenv('IMPORT_BUCKET', 'default-bucket-name')

def import_to_neptune(file): 
    try:
        d_frame = pd.read_csv(file)
        print(f"Data loaded from {file}")
    except FileNotFoundError:
        print("File not found.")
        exit(1)
    except pd.errors.EmptyDataError:
        print("The provided file is empty. Please load appropriate file.")
        exit(1)

    node_id = input("Which column would you like to use as the unique identifier for the nodes? (Case-sensitive!): ")
    if node_id not in d_frame.columns:
        print(f"{node_id} not found in data.")
        exit(1)

    if d_frame[node_id].isnull().any():
        print("The column you identified contains null values. Please rectify.")
        exit(1)

    property_id = input("What are the column names you would like to use as the properties for your nodes?: ")
    if property_id not in d_frame.columns:
        print(f"{property_id} not found in data.")
        exit(1)

    relationship_id = input("Enter the column names that represent relationships, no name is fine: ")
    relationship_id = [rel.strip() for rel in relationship_id.split(',') if rel.strip()]

    vertex_export = "/home/ec2-user/vertex.csv"
    edges_export = "/home/ec2-user/edges.csv"

    with open(vertex_export, 'w', newline='') as v, open(edges_export, 'w', newline='') as e:
        vertex_mod = csv.writer(v)
        edge_mod = csv.writer(e)

        edge_id = 1

        vertex_mod.writerow(['~id', '~label'] + relationship_id)
        edge_mod.writerow(['~id', '~from', '~to', '~label'])

        for index, row in d_frame.iterrows():
            node_id_trace = f'{row[node_id]}'
            node_properties = [row[col] for col in property_id if col in d_frame.columns]
            vertex_mod.writerow([node_id, 'Node'] + node_properties)

            for rel_col in relationship_id:
                if rel_col in d_frame.columns and pd.notna(row[rel_col]):
                    related_node_id = f'{row[rel_col]}'
                    edge_mod.writerow([f'e{edge_id}', node_id, related_node_id, f'relatedTo_{rel_col.strip()}'])
                    edge_id += 1

    print('Vertices and edges CSV files have been written.')

    client_container = boto3.client('s3') 

    try: 
        client_container.upload_file(vertex_export, IMPORT_BUCKET, 'vertex.csv')
        print(f"{vertex_export} uploaded to S3 bucket: {IMPORT_BUCKET}")
        client_container.upload_file(edges_export, IMPORT_BUCKET, 'edges.csv')
        print(f"{edges_export} uploaded to S3 bucket: {IMPORT_BUCKET}")
    except boto3.exceptions.S3UploadFailedError as aws_err:
        print(f"Upload error: {aws_err}")
    except boto3.exceptions.NoCredentialsError as auth_err:
        print(f"Credentials error: {auth_err}")

    for s3_uri in [f's3://{IMPORT_BUCKET}/vertex.csv', f's3://{IMPORT_BUCKET}/edges.csv']:
        nep_url = f"https://{NEP_ENDPOINT}:8182/loader"

        payload_delivery = {
            "source": s3_uri,
            "format": "csv",
            "iamRoleArn": IAM_ROLE_ARN,
            "region": "us-east-1",
            "failOnError": "TRUE",
        }

        headers = {
            "Content-Type": "application/json"
        }

        try:
            response = requests.post(nep_url, data=json.dumps(payload_delivery), headers=headers, verify=False)
            response.raise_for_status()
            return response.json()
        except requests.exceptions.RequestException as err:
            if response is not None:
                print(f"Error loading data to Neptune for {s3_uri}: {e}, Response details: {response.text}")
            else:
                print(f"Error loading data to Neptune for {s3_uri}: {e}")


import boto3
import pandas as pd
from decimal import Decimal

dynamodb = boto3.resource('dynamodb', region_name='us-east-1') #authenticates with iam role
client = dynamodb.meta.client

def float_to_dec(value):
    if isinstance(value, float):
        return Decimal(value)
    return value

def import_to_dynamodb(file, table_name, primary_key):
    #check if table exists
    tables = client.list_tables()['TableNames']
    if table_name in tables:
        print(f"Table {table_name} already exists, inserting data now ...")
        table = dynamodb.Table(table_name)
    #create table
    else:
        try: 
            table = dynamodb.create_table(
                TableName=table_name,
                KeySchema=[
                    {
                        'AttributeName': primary_key,
                        'KeyType': 'HASH'
                    }
                ],
                AttributeDefinitions=[
                    {
                        'AttributeName': primary_key,
                        'AttributeType': 'S'
                    }
                ],
                ProvisionedThroughput={
                    'ReadCapacityUnits': 1,
                    'WriteCapacityUnits': 1
                }
            )
            table.wait_until_exists()
            print(f"Table {table_name} created successfully")
        except Exception as e:
            print(f"Error creating table: {e}")
            return None
    
    #import rows into table 
    data = pd.read_csv(file)
    for _, row in data.iterrows():
        item = {k: float_to_dec(v) for k, v in row.to_dict().items()}
        try:
            table.put_item(Item=item)
        except Exception as e:
            print(f"Error importing data: {e}")
            return None
        
    print('Data has been imported to DynamoDB')






from aws_databridge.utils.convert_to import text_to_csv, json_to_csv, xml_to_csv, csv_to_json
from aws_databridge.db_import.rds import import_to_rds
from aws_databridge.db_import.dynamodb import import_to_dynamodb
from aws_databridge.db_import.document import import_to_documentdb
from aws_databridge.db_import.neptune import import_to_neptune
import boto3
import os

def main():
    import_path = input('Do you want to import data from your ec2 or s3? (ec2 = 1, s3 = 2): ')
    if import_path == '1':
        file = input('Enter the full name of the file (e.g. animals.txt , include folder path if applicable-> folder/animals.txt):')
        file = os.path.join('/home/ec2-user/', file)
        file_type = file.split('.')[1]
    elif import_path == '2':
        bucket = input('Enter the name of the bucket you would like to import data from: ')
        file = input('Enter the full name of the file (e.g. animals.txt): ')
        file_type = file.split('.')[1]
        s3 = boto3.client('s3')
        local_file_path = os.path.join('/home/ec2-user/', file) 
        s3.download_file(bucket, file, local_file_path )
        file = local_file_path
        file_type = file.split('.')[1]


    db_choice = input('What database would you like to import this data to? (rds = 1, dynamo = 2, neptune = 3, documentDB = 4): ')
    table_name = input('Enter the name of the table you would like to import this data to: ')

   # import data to the selected database, allowing for multiple imports if neccessary 
    stop = 0;
    while stop == 0:

        #convert supported non csv types to csv
        if file_type != 'csv':
            if file_type == 'txt':
                print('Converting data to csv...')
                file = text_to_csv(file)
                print('Data has been converted to csv')
                file_type = 'csv'
            elif file_type == 'json' and db_choice != '3':
                print('Converting data to csv...')
                file = json_to_csv(file)
                print('Data has been converted to csv')
                file_type = 'csv'
            elif file_type == 'xml':
                print('Converting data to csv...')
                file = xml_to_csv(file)
                print('Data has been converted to csv')
                file_type = 'csv'
        if db_choice == '3' and file_type == 'csv':
            print('Converting data to json...')
            file = csv_to_json(file)
            print('Data has been converted to json')
            file_type = 'json'

        if db_choice == '1':
            import_to_rds(file, table_name)
        elif db_choice == '2':
            primary_key = input('Enter the primary key for the table: ')
            import_to_dynamodb(file, table_name, primary_key)
        elif db_choice == '3':
            import_to_neptune(file)
            print('Data has been imported to Neptune')
        elif db_choice == '4':
            import_to_documentdb(file)
            print('Data has been imported to DocumentDB')
        stop = int(input('Would you like to import this file to another database? (yes = 0, no = 1): '))
        if stop == 0:
            db_choice = input('What database would you like to import this data to? (rds = 1, dynamo = 2, neptune = 3, documentDB = 4): ')
            table_name = input('Enter the name of the table you would like to import this data to: ')

    # Delete the S3 file after processing
    if import_path == '2':
        try:
            os.remove(file)
            print(f"File '{file}' has been deleted from the EC2 instance.")
        except FileNotFoundError:
            print(f"File '{file}' not found.")
        except Exception as e:
            print(f"Error deleting file: {e}")

if __name__ == '__main__':
    main()


import csv
import pandas as pd
import json

def text_to_csv(filepath):
    
    #test
    # test_text = """name age city
    # Jay 25 New_York
    # Abahn 30 Vegas
    # Matt 22 connecticut
    # name 28 Houston
    # """
    # with open("test_data.txt", "w") as file:
    #     file.write(test_text)
    #     filepath = "test_data.txt"

    with open(filepath, 'r') as file:
        lines = file.readlines()

    a_names = lines[0].split() #attribute names
    rows = [line.split() for line in lines[1:]] #tuple values

    new_filepath = filepath.replace('txt', 'csv')
    with open(new_filepath, 'w', newline='') as file:
        writer = csv.writer(file)
        writer.writerow(a_names)
        writer.writerows(rows)
    
    return new_filepath

def json_to_csv(filepath):
    #test
    # test_json = [
    #     {"name": "Jay", "age": 25, "city": "New York"},
    #     {"name": "Abahn", "age": 30, "city": "Vegas"},
    #     {"name": "Matt", "age": 22, "city": "connecticut"},
    #     {"name": "name", "age": 28, "city": "Houston"}
    # ]
    # with open("test_data.json", "w") as file:
    #     json.dump(test_json, file, indent=4)
    #     filepath = "test_data.json"
    data = pd.read_json(filepath)
    new_filepath = filepath.replace('json', 'csv')
    data.to_csv(new_filepath, index=False)

    return new_filepath

def xml_to_csv(filepath):

    # test 
    # test_xml = """<?xml version="1.0"?>
    # <data>
    #     <person>
    #         <name>Jay</name>
    #         <age>25</age>
    #         <city>New York</city>
    #     </person>
    #     <person>
    #         <name>Abahn</name>
    #         <age>30</age>
    #         <city>Vegas</city>
    #     </person>
    #     <person>
    #         <name>Matt</name>
    #         <age>22</age>
    #         <city>connecticut</city>
    #     </person>
    #     <person>
    #         <name>name</name>
    #         <age>28</age>
    #         <city>Houston</city>
    #     </person>
    # </data>"""

    # # Write the XML to a file
    # with open("test_data.xml", "w") as file:
    #     file.write(test_xml)
    #     filepath = "test_data.xml"

   data = pd.read_xml(filepath)
   new_filepath = filepath.replace('xml', 'csv')
   data.to_csv(new_filepath, index=False)
    
   return new_filepath

def csv_to_json(filepath):
    data = pd.read_csv(filepath)
    new_filepath = filepath.replace('csv', 'json')
    data.to_json(new_filepath, orient='records', indent=4)
    
    return new_filepath





from setuptools import setup, find_packages

setup(
    name="aws-databridge",
    version="2.0.1",
    packages=find_packages(where='src'),
    package_dir={'': 'src'},
    include_package_data=True,
    install_requires=[
       "boto3>=1.20.0",  
        "pandas",
        "pymysql",
        "lxml"
    ],
    entry_points={
        "console_scripts": [
           "aws-databridge=aws_databridge.main:main",
        ]
    },
    author="Jay, Aban, Matt",
    description="An AWS EC2 CLI application that supports XML, TXT, JSON, and CSV imports to various AWS databases.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/Jaysalgado/aws-databridge",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)



import csv 
import os 
import pymysql

#rds credentials stored in ec2
RDS_HOST = os.getenv("RDS_HOST")
RDS_USER = os.getenv("RDS_USER")
RDS_PASSWORD = os.getenv("RDS_PASSWORD")
RDS_DB = os.getenv("RDS_DB")
RDS_PORT = int(os.getenv("RDS_PORT", 3306))

def import_to_rds(filepath, table_name):
    #connect to rds 
    try:
        conn = pymysql.connect(
            host=RDS_HOST,
            user=RDS_USER,
            password=RDS_PASSWORD,
            port=RDS_PORT
        )
    except pymysql.MySQLError as e:
        print(f"Error connecting to RDS: {e}")
        return None 
    
    #read data from file
    with open(filepath, 'r') as file:
        csv_reader = csv.DictReader(file)
        a_names = csv_reader.fieldnames
        rows = [list(row.values()) for row in csv_reader]

    #database object (cursor) to interact with the database
    with conn.cursor() as cur:
        # Create the database if it doesn't exist
        cur.execute(f"CREATE DATABASE IF NOT EXISTS {RDS_DB}")
        conn.close()

    #redo the connection with the database specified 
    try:
        conn = pymysql.connect(
            host=RDS_HOST,
            user=RDS_USER,
            password=RDS_PASSWORD,
            db=RDS_DB,
            port=RDS_PORT,
            autocommit=True #prevents changes from being temporary
        )
    except pymysql.MySQLError as e:
        print(f"Error connecting to RDS: {e}")
        return None 
    
    with conn.cursor() as cur:
        # Dynamically create the table based on the CSV headers
        placeholders = ', '.join(f"{col} VARCHAR(255)" for col in a_names)  # Dynamic column creation
        cur.execute(f"CREATE TABLE IF NOT EXISTS {table_name} ({placeholders})")
            
        # Insert the data -> VALUES (%s, %s, %s, ...)
        insert_query = f"INSERT INTO {table_name} VALUES ({', '.join(['%s'] * len(a_names))})"
        cur.executemany(insert_query, rows)

    conn.close()

    print('Data has been imported to RDS')